/* This script creates lines from a point layer, or
 * creates multipart lines from a line layer.
 * It creates new shapes by grouping existing shapes
 * by a given attribute. All consecutive shapes with
 * the same group-id will be the base for each new
 * shape, in the order they appear in the attribute
 * view. The result is a new layer.
 * This script requires ShapeUp version 1.3.02 or later.
 * Copyright (c) Niklas Lindquist, 2008
 */
function copyAttributes(shpTarget, shpSource)
{
	if (shpTarget == null)
		return;

	var attrs = shpSource.attributeCount;
	
	for (var i = 0; i < attrs; ++i)
	{
		shpTarget.setAttribute(i, shpSource.getAttribute(i));
	}
}

function createPolyPolylinesFromPolylines(sourceLayer, targetLayer, idcol)
{
	for (var i = 0; i < sourceLayer.shapeCount; ++i)
	{
		var shpMaster = sourceLayer.getShape(i);
		var id = shpMaster.getAttribute(idcol);
		var id2 = id;
		var shpNew = targetLayer.createShape(shpMaster);
		copyAttributes(shpNew, shpMaster);
		++i;

		for ( ;i < sourceLayer.shapeCount; ++i)
		{
			var shpSlave = sourceLayer.getShape(i);
			id2 = shpSlave.getAttribute(idcol);
			if (id2 == id)
			{
				for (var part = 0; part < shpSlave.partCount; ++part)
				{
					var arr = new Array();
					var pointCount = shpSlave.getCoordCount(part);
					for (var c = 0; c < pointCount; ++c)
					{
						arr[arr.length] = shpSlave.getCoord(part, c);
					}
					shpNew.addPart(arr);
				}
			}
			else
			{
				--i;
				break;
			}
		}
	}
}


function createPolylinesFromPoints(sourceLayer, targetLayer, idcol)
{
	for (var i = 0; i < sourceLayer.shapeCount; ++i)
	{
		var shpMaster = sourceLayer.getShape(i);
		var arr = new Array();
		var id = shpMaster.getAttribute(idcol);
		var id2 = id;

		for ( ; id == id2 && i < sourceLayer.shapeCount; ++i)
		{
			id2 = sourceLayer.getShape(i).getAttribute(idcol);
			if (id2 == id)
				arr[arr.length] = sourceLayer.getShape(i).getCoord(0, 0);
			else
			{
				var shp = targetLayer.createShape(arr);
				copyAttributes(shp, shpMaster);
				--i;
				break;
			}
		}
		if (i == sourceLayer.shapeCount)
		{
			var shp = targetLayer.createShape(arr);
			copyAttributes(shp, shpMaster);
		}
	}
}


function run()
{
	var source = workspace.browseLayer('Select source layer');
	if (source == null)
		return;
	if (source.shapeCount <= 1)
	{
		alert('Not enough shapes in source theme to calculate compound.');
		return;
	}

	var idcol = source.browseAttribute('Select grouping attribute');
	if (idcol == -1)
		return;

	if (source.getShape(0).type == 'Point')
	{
		var target = workspace.addNewLayer('Polyline', 'Polylines', source);
		createPolylinesFromPoints(source, target, idcol);
	}
	else
	{
		var target = workspace.addNewLayer('Polyline', 'Multi-part Polylines', source);
		createPolyPolylinesFromPolylines(source, target, idcol);
	}

	workspace.refresh();
}

run();

